#################################################################################################################
#
# Copyright (C) 2013-2015 X2Go Project - http://wiki.x2go.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Copyright (C) 2013-2015 Guangzhou Nianguan Electronics Technology Co.Ltd. <opensource@gznianguan.com>
# Copyright (C) 2013-2015 Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
#
#################################################################################################################
package X2Go::Telekinesis::CoreSupport;
use strict;
#use warnings;
use File::stat;
use Exporter qw(import);

our @EXPORT_OK = qw( vCompChkWeNewest versionCompCheck getConnectToken premZealot getCmdFlags dbugPrint tcDie sanitizeDirPath sanitizeFilePath sanitizer checkPID genRandSID genRandToken clups);


sub vCompChkWeNewest {
 my $We   = $_[0];
 my $Them = $_[1];
 if (($We =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/) and ($Them =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/)) {
   my ($w1,$w2,$w3,$w4) = $We   =~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/;
       $w1 = sprintf("%03d", $w1);$w2 = sprintf("%03d", $w2);$w3 = sprintf("%03d", $w3);$w4 = sprintf("%03d", $w4);
   my $chkWe = "$w1$w2$w3$w4";
   my ($t1,$t2,$t3,$t4) = $Them =~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/;
       $t1 = sprintf("%03d", $t1);$t2 = sprintf("%03d", $t2);$t3 = sprintf("%03d", $t3);$t4 = sprintf("%03d", $t4);
   my $chkThem = "$t1$t2$t3$t4";
   if ($chkWe >= $chkThem) {
     return "U";
   } elsif ($chkWe < $chkThem) {
     return "T";
   } else {
     return "WTF";
     dbugPrint("vCompChkWeNewest did not return valid data!");
   }
 } else {return 0;}

}

sub versionCompCheck {
  my $inCheckDir = sanitizeDirPath($_[0]);
  my $inCheckType = sanitizer("num",$_[1]);
  my $inCheckApp = sanitizer("anumazlc",$_[2]);
  my $inCheckVer = sanitizer("numdot",$_[3]);

  if (-d $inCheckDir) { # If the dir doesn't exist there's no point in doing anything else?
    my $useCheckDir;
    if ($inCheckApp eq "self") {# <- If we are looking up information about our self...
      $useCheckDir = $inCheckDir;
     } else {  # Setting up the appropriate base dir depending on self/app lookup..
      $useCheckDir = "$inCheckDir/apps"; # <- If we are looking for version info for external applications or features...
    }
    if (-f "$useCheckDir/$inCheckApp.vc") {# <- good idea to check if the ".vc" file exists before we try to do anything else.
      open(VCF,"$useCheckDir/$inCheckApp.vc");
      my ($vcfOwnVersionA,@vcfCompInfo) = <VCF>;
      close(VCF);
      my ($vcfOwnVersion,$isItSelf) = $vcfOwnVersionA =~ /^([0-9\.]*)\:(.)/;# <- First line should be the locally installed version (what we got our self... hence the S)
      $isItSelf = clups($isItSelf);
      if ($isItSelf eq "S") { # <- and this should be S if not... the .vc file is not a good .vc file....
        if ($inCheckType eq 0) {# Check type "0" is a local version request call...
          return ($inCheckApp,$vcfOwnVersion);#so we answer the call with this reply
        } elsif (($inCheckType eq 1) and ($inCheckVer =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/)) {# Type 1 is a compatibility check call where we reply 0 or 1
          if ($inCheckVer eq $vcfOwnVersion) {# If check version is the same as our version... the it should be ok.. right?!!
            return 1;
          } else {# Otherwise start more "in-depth" checking procedures....
            my $vCompState = "N";

            foreach my $vcfCompLine (@vcfCompInfo) {# the rest of the .vc file should be lines with version numbers and compatibility flags.
              my ($vcfCLver,$vcfCLstate) = split(/\:/,uc($vcfCompLine));
              $vcfCLver = sanitizer("numdot",$vcfCLver);
              $vcfCLstate =~ s/[^CB]//g;  # flags are either C=compatible or B=broken..
              if (($vcfCLstate) and ($vcfCLver =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/)) {# the .vc file line to be a valid line the state flag need to be valid so does the version number format....
                if ($inCheckVer eq $vcfCLver) {
                  unless ($vCompState eq "B") {$vCompState = $vcfCLstate;}# If for some nutty reason there are duplicates and one is broken... then thats what we'll report
                }
              }
            }
            # If the final judgement is C (compatible) then we return 1, for anything else we return 0)
            if ($vCompState eq "C") {return 1;} else {return 0;}
          }
        } else {return 0;}
      } else {return 0;}
    } else {return 0;}
  } else {return 0;}
}


sub premZealot {
  my $thePath = $_[0];$thePath =~ s/\n//g;
  if (-f $thePath) {
    chmod(0600,$thePath);
    my $fStat = stat($thePath);
    if (getpwuid($fStat->uid) ne $ENV{'USER'}) {tcDie("Ownership/security issue!\nYou don't own \"$thePath\"!!!");}
    if ($fStat->mode ne 33152) {tcDie("Ownership/security issue!\nFailed to set the premissions we want on\n \"$thePath\"\n");}
  } elsif (-d $thePath) {
    chmod(0700,$thePath);
    my $fStat = stat($thePath);
    if (getpwuid($fStat->uid) ne $ENV{'USER'}) {tcDie("Ownership/security issue!\nYou don't own \"$thePath\"!!!");}
    if ($fStat->mode ne 16832) {tcDie("Ownership/security issue!\nFailed to set the premissions we want on\n \"$thePath\"\n");}
  } elsif (-S $thePath) {
    chmod(0700,$thePath);
    my $fStat = stat($thePath);
    if (getpwuid($fStat->uid) ne $ENV{'USER'}) {tcDie("Ownership/security issue!\nYou don't own \"$thePath\"!!!");}
    if ($fStat->mode ne 49600) {tcDie("Ownership/security issue!\nFailed to set the premissions we want on\n \"$thePath\" (",$fStat->mode,")\n");}
  } else {
    tcDie("Ownership/security issue!\nFailed to set the premissions we want on:\n \"$thePath\"\n\npremZealot called on something we could not recognize!\n");
  }
}


sub getCmdFlags {
  my %theArgs;
  foreach my $ent (@_) {
    $ent =~ s/\n//g;
#print "EX:$ent:EX\n";
    if ($ent =~ /^\-set([A-Z0-9]{4,24})\=(.*)$/) {
      $theArgs{$1} = $2;
    }
  }
  return %theArgs;
}


sub dbugPrint {
  if (($::CmdFlags{'DEBUG'}) or (uc($_[1]) eq "T")) {
    my $lTime = time();
    my $logLevel = 0;
    if (uc($_[1]) eq "T") {$logLevel = 1;}
    elsif (uc($_[1]) eq "W") {$logLevel = 2;}
    elsif (uc($_[1]) eq "I") {$logLevel = 9;}
    if (-d $::X2GoSesHome) {
      open(LOG,">>$::X2GoSesHome/tekises.log") or die("Can't write to logfile: $::X2GoSesHome/tekises.log");
      foreach my $logLine (split(/\n/,$_[0])) {
        if (length(clups($logLine)) > 0) {
          print LOG "$lTime|$logLevel:$logLine\n";
        }
      }
      close(LOG);
    }
  }
}

sub tcDie {
  my $error = "@_";
  dbugPrint($error,"T");
  die($error);#DIE DIE DIE!!!!!!!!!!!
}

sub getConnectToken {
  my $TokenType = lc($_[0]);
  my $TokenHomeDir = $_[1];
  if (($TokenType eq "remote") or ($TokenType eq "local")) {
    if (-f "$TokenHomeDir/$TokenType/token") {
      open(TOKEN,"$TokenHomeDir/$TokenType/token");my ($token,undef) = <TOKEN>;close(TOKEN);
      $token = clups($token);
      return $token;
    } else {return 0;tcDie("NO CONNECTION TOKEN!?!?!!");}
  } else {return 0;tcDie("NO CONNECTION TOKEN!?!?!!");}
}

sub sanitizeDirPath {
# NOT REALLY WORTH A DAMN, SOMEONE SHOULD FIX THIS!!!
  my $fPath = $_[0];
  $fPath =~ s/\;//g;
  if (-d $fPath) {
    if ($fPath  =~ /^(.*)$/) {
      $fPath = $1;
      return $fPath;
    } else {return 0;}
  } else {return 0;}
}

sub sanitizeFilePath {
# NOT REALLY WORTH A DAMN, SOMEONE SHOULD FIX THIS!!!
  my $fPath = $_[0];
  $fPath =~ s/\;//g;
  if (-f $fPath) {
    if ($fPath  =~ /^(.*)$/) {
      $fPath = $1;
      return $fPath;
    } else {return 0;}
  } else {return 0;}
}

sub sanitizer {
# Overzealous string sanitizer that makes perl strict and  perl -T happy...
  my $type   = $_[0];
  my $string = $_[1];
  if ($type eq "anumazcs") {
    $string =~ s/[^a-zA-Z0-9]//g;
    if ($string =~ /^([a-zA-Z0-9]*)$/) {
      $string = $1;
      return $string;
    } else {return 0;}
  } elsif ($type eq "alpazuc") {
    $string = uc($string);
    $string =~ s/[^A-Z]//g;
    if ($string =~ /^([A-Z]*)$/) {
      $string = $1;
      return $string;
    } else {return 0;}
  } elsif ($type eq "alpazlc") {
    $string = lc($string);
    $string =~ s/[^a-z]//g;
    if ($string =~ /^([a-z]*)$/) {
      $string = $1;
      return $string;
    } else {return 0;}
  } elsif ($type eq "anumazlc") {
    $string = lc($string);
    $string =~ s/[^a-z0-9]//g;
    if ($string =~ /^([a-z0-9]*)$/) {
      $string = $1;
      return $string;
    } else {return 0;}
  } elsif ($type eq  "num") {
    $string =~ s/\D//g;
    if ($string =~ /^([0-9]*)$/) {
      $string = $1;
      return $string;
    } else {return 0;}
  } elsif ($type eq  "numdot") {
    $string =~ s/[^0-9\.]//g;
    if ($string =~ /^([0-9\.]*)$/) {
      $string = $1;
      return $string;
    } else {return 0;}
  } elsif ($type eq "anumazcsdaus") {
    $string =~ s/[^a-zA-Z0-9\_\-]//g;
    if ($string =~ /^([a-zA-Z0-9\_\-]*)$/) {
      $string = $1;
      return $string;
    } else {return 0;}
  } elsif ($type eq "anumazcsdausdots") {
    $string =~ s/[^a-zA-Z0-9\_\-\:\.]//g;
    if ($string =~ /^([a-zA-Z0-9\_\-\:\.]*)$/) {
      $string = $1;
      return $string;
    } else {return 0;}
  } elsif ($type eq "anumazcsdausdotspipe") {
    $string =~ s/[^a-zA-Z0-9\_\-\:\.\|]//g;
    if ($string =~ /^([a-zA-Z0-9\_\-\:\.\|]*)$/) {
      $string = $1;
      return $string;
    } else {return 0;}
  } elsif ($type eq "pnixusername") {
    $string =~ s/[^a-zA-Z0-9\_\-\.]//g;
    if ($string =~ /^([a-zA-Z0-9\_\-\.]*)$/) {
      $string = $1;
      return $string;
    } else {return 0;}
  } elsif ($type eq "x2gosid") {
          $string =~ s/[^a-zA-Z0-9\_\-\$\.]//g;
          if ($string =~ /^([a-zA-Z0-9\_\-\$\.]*)$/) {
                  $string = $1;
                  if ($string =~ /^([a-zA-Z\_][a-zA-Z0-9\_\-\.]{0,31}[\$]?)\-([\d]{2,4})\-([\d]{9,12})\_[a-zA-Z0-9\_\-]*\_dp[\d]{1,2}$/) {
                          if ((length($1) > 0) and (length($1) < 32)){
                                  return $string;
                          } else {return 0;}
                  } else {return 0;}
          } else {return 0;}
  } elsif ($type eq "SOMETHINGELSE") {
    return 0;
  } else {tcDie("Some dumb smuch asked for a non-existent sanitation rule....\nIt may be a big deal... or it may not be...\nNevertheless, we're putting an end to this...");}
}

sub checkPID {
  my $inpid = sanitizer("num",$_[0]);
  open(PS,"/bin/ps --no-headers -o %u,%p,%n,%c -p $inpid|");
  my ($pidInf,undef) = <PS>;
  close(PS);
  my ($user,$pid,$nice,$cmd)  = split(/\,/,clups($pidInf));
  $pid =~ s/\D//g;
  return ($pid,$user,$nice,$cmd)
}

sub genRandSID {
  srand();
  my $time = time();
  my $rcnt = (16 - length($time));
  my @chars = ('0'..'9','a'..'z','A'..'Z');
  my $randmake;
  for (my $i=0; $i<$rcnt; $i++) {$randmake .= $chars[int(rand($#chars + 1))];}
  return clups("$time$randmake");
}


sub genRandToken {
  my $RemOrLocal = $_[0];
  my $HomeDir = $_[1];
  my $FullTokenPath;
  if ($RemOrLocal eq "local") {$FullTokenPath = "$HomeDir/local/token";}
  elsif ($RemOrLocal eq "remote") {$FullTokenPath = "$HomeDir/remote/token";}
  else {tcDie("Someone stepped on a turd and wiped it off, somewhere in the code of this thig...");}
  srand();
  my $rcnt = 128;
  my @chars = ('0'..'9','a'..'z','A'..'Z');
  my $randmake;
  for (my $i=0; $i<$rcnt; $i++) {$randmake .= $chars[int(rand($#chars + 1))];}
  open(TOKEN,">$FullTokenPath") or tcDie("Error while trying to save token info....\nWe cant go on like this...\n\n BYE!");
  print TOKEN "$randmake\n";close(TOKEN);
  premZealot($FullTokenPath);
  return $randmake;
}


sub clups {my $string = "@_";$string =~ s/\n//g;$string =~ s/\ //g;$string =~ s/\s//g;return $string;}




1;
